	.include "tnc.h"
	.area code (REL)
;       include ib.mac          ;tty interrupt catchers

;---------------------------------------------------------------------------
; we get here whenever -cts, -dcd or -sync inputs change, as well as break
; detection. since -dcd
; is always tied to +5 volts, we need only worry about -cts and -sync.
; -cts is wired to pin 20, dtr, of the rs232 connector, and is supposed to
; be used for host to tnc handshaking; we ignore this transition (we assume
; that the host is always ready).  we also ignore break detection.  we are
; only interested in -sync transitions, so we can keep time.
; note!  this is the only routine that is allowed to use the other reg set!!
; deal with break detection...
;
; [DIX] This is not the only routine that uses the alt. reg. set, and in fact 
; it only uses the alt. AF pair.
;
; [DIX] Since in DIXTNC periodic interrupts are generated by the Z80-CTC
; at the correct rate of 100 Hz much of the original stuff has been deleted.
; ib_ext only deals with break detection. See i_ctc for timer interrupt.

	.globl ib_ext
ib_ext: ex af,af'
        ld a,#0x10		;reset Ext/Status interrupts
        out (b_ctl),a
        in a,(b_ctl)		;grab RR0
        and #break_abort	;Check if we are doing a break/abort think
        jr z,ib_nba		;There is no break/abort
; Else break/abort bit on, note state change
        ld a,#true
        ld (in_break),a
        in a,(b_dat)		;clear out any null character from buffer
        jr ib_ok
ib_nba: ; if no break/abort check if we are in break/abort state
        ld a,(in_break)
        or a
        jr z,ib_ok		;Nothing goin on, break OK
;Else we were in break mode, and this is the tail end of a break
        xor a
        ld (in_break),a
        in a,(b_dat)		;discard the single extraneous null
ib_ok:  ex af,af'
        ei
        reti


;---------------------------------------------------------------------------
	.globl ib_special
ib_special:
	push	af
	ld	a,#0x30		; error reset
	out	(b_ctl),a
	pop	af
	ei
	reti

;---------------------------------------------------------------------------
; the tx has become empty, shove a new character out
	.globl ib_tbe
ib_tbe:
        ex      af,af'                  ; new char will return in a
        exx

	ld	a,(out_esc_mode)
	or	a
        jr      z,ib_t1                 ; not escaped, so go here
; else we are escaped, so send escaped char
        ld      a,(out_char)            ; char which follows escape
	or	a
        jr      z,ib_t2 ; special case if at end of frame, clean up
	out	(b_dat),a
	xor	a
	ld	(out_esc_mode),a	; get out of escaped mode
        jr      ib_t9                   ; all for now...
ib_t1:
        ld      hl,(out_ch_head)        ; we are currently on this buffer,
	call	getchar			; as getchar() needs to know
        ld      (out_ch_head),hl        ; maybe hl changed,so save it in case

        jr      z,ib_tdone              ; if no more chars, deal with this
	cp	#fesc
        jr      z,ib_t1a                ; deal with fesc char in data stream
	cp	#fend
        jr      z,ib_t1b                ; deal with fend char in data stream
; else this char is nothing special, so shove it out

        out     (b_dat),a               ; shove it out
        jr      ib_t9                   ; if this is not last char, all for now

; else this is last char, send fend
ib_tdone:
	ld	a,#fend
	out	(b_dat),a
	ld	a,#true
	ld	(out_esc_mode),a	; set special escaped mode by...
	xor	a
	ld	(out_char),a		;... making escaped char a 0
        jr      ib_t9                   ; all till tx buffer goes empty again.

; here if are completely done sending frame
ib_t2:
	ld	hl,(out_head_cbuf)
	inc	hl
	inc	hl		
	ld	de,#out_bottom
        ex      de,hl
        or      a                       ;clear carry
        sbc     hl,de
        ex      de,hl

        jr      nz,ib_t2a               ; yes it is!

	ld	hl,#out_top		; else, make a circular buffer
ib_t2a:
	ld	(out_head_cbuf),hl	; we will work on this one next
	xor	a
	ld	(out_started),a		; not doing outputs anymore
	ld	(out_esc_mode),a	; !! not in escaped mode anymore !!

	ld	a,#0x28			; needed for async
	out	(b_ctl),a		; reset tx interrupt pending

ib_t9:
        exx
        ex      af,af'
	ei
	reti				; now get our butts out of here...

; here is fesc in data stream
ib_t1a:
	out	(b_dat),a		; ship fesc character to port
	ld	a,#tfesc			; ready what will be next char
ib_t1z:
	ld	(out_char),a		; set char for next time
	ld	a,#true
	ld	(out_esc_mode),a	; we are in escaped mode
        jr      ib_t9           ; all for now

; here is fend in data stream
ib_t1b:
	ld	a,#fesc
	out	(b_dat),a
	ld	a,#tfend
        jr      ib_t1z          ; rest is same as fesc case


;---------------------------------------------------------------------------
; got a char from the tty port, deal with it.

	.globl ib_rca
ib_rca:
        ex      af,af'
        exx

	in	a,(b_ctl)	; read rr0; force reg pointer to be 0
	ld	a,#1
	out	(b_ctl),a	; ready to read rr1
	in	a,(b_ctl)	; grab rr1
	and	#framing_error	; isolate the fe bit
        jr      z,ib_rtop       ; no framing error, so process this char

; else we have a framing error - ignore this char & flush this frame...
	call	sta_off		; off with the led!
	in	a,(b_dat)	; flush erroneous character
	xor	a
	ld	(in_state),a	; force receiver to look for fend
        ld      a,(in_allo_buf)
	or	a
        jr      z,ib_rc9        ; if no buffer is allocated, done; exit.

; else we were receiving a data slip frame, so flush it.
	ld	hl,(in_head)
	call	free_chain	; dump these buffers back to free list
        jr      ib_rc9  ; and get out of here!

ib_rtop:
	ld	a,(in_state)	; get our state machine value
	or	a
        jr      z,ib_r0 ; in state 0, waiting for fend
	cp	#1
        jr      z,ib_r1 ; in state 1, saw fend
	cp	#2
        jr      z,ib_r2 ; in state 2, data to follow
	cp	#3
        jp      z,ib_r3 ; saw fesc, expecting tfesc or tfend
	cp	#10
        jp      z,ib_r10        ; expecting txdelay
	cp	#20
        jp      z,ib_r20        ; expecting p value
	cp	#30
        jp      z,ib_r30        ; expecting slottime value
	cp	#40
        jp      z,ib_r40        ; expecting tailtime value
	cp	#50
        jp      z,ib_r50        ; expecting full/half duplex value

;else we don't know what happened, ignore it.
ib_rcjunk:
	in	a,(b_dat)
ib_rcfend:
	xor	a
ib_rcstate:
	ld	(in_state),a	;go into in_state 0, fend hunt
ib_rc9:
        exx
        ex      af,af'          ; throw it away, we don't need junk
	ei
	reti

; here if we are hunting for fend character
ib_r0:
	call	sta_off

	in	a,(b_dat)
	cp	#fend
        jr      nz,ib_rc9       ; if we didn't see an fend, keep looking

; else is an fend, change state
	ld	a,#1
        jr      ib_rcstate

; get here if we've seen fend character; look for command byte
ib_r1:
	call	sta_off
	in	a,(b_dat)
	cp	#fend
        jr      z,ib_rc9        ; just another fend, keep looking for cmd

	call	sta_on		;getting valid slip; show in sta led

; here if we do not have an fend (expecting command byte)

	and	#0x0f
        jr      z,ib_r1a        ; 0 command means data will follow
	cp	#1
        jr      z,ib_r1b        ; 1 command means txdelay will follow
	cp	#2
        jr      z,ib_r1c        ; 2 command means p(persistence) will follow
	cp	#3
        jr      z,ib_r1d        ; 3 command means slot time will follow
	cp	#4
        jr      z,ib_r1e        ; 4 command means tailtime to follow
	cp	#5
        jr      z,ib_r1f        ; 5 command means full/half duplex to come

; here if we receive bogus command byte, flush rest of frame

	call	sta_off		;bogosity, so turn off sta led

        jr      ib_rcfend

; data are expected, change state
ib_r1a:
	ld	a,#2
        jr      ib_rcstate

; txdelay to follow, change state
ib_r1b:
	ld	a,#10
        jr      ib_rcstate

; p to follow, change state
ib_r1c:
	ld	a,#20
        jr      ib_rcstate

; slottime to follow, change state
ib_r1d:
	ld	a,#30
        jr      ib_rcstate

; tailtime to follow, change state
ib_r1e:
	ld	a,#40
        jr      ib_rcstate

; full/half duplex to follow, change state
ib_r1f:
	ld	a,#50
        jr      ib_rcstate

; these bytes are data
ib_r2:
	in	a,(b_dat)
	cp	#fend
        jr      z,ib_r2b        ; fend means to queue this buffer
        ld      b,a             ; save the char we read for a bit..

        ld      a,(in_allo_buf)
	or	a
        jr      nz,ib_r2c       ; if we already allocated buffer

        call    allo_buf        ; get our initial buffer to mess with
        jr      z,ib_rcfend     ; if no room, flush this frame

ib_r22:
	ld	a,#true
        ld      (in_allo_buf),a ; make ourselves active

	ld	(in_buffer),hl
	ld	(in_head),hl	; save current & head of chain pointers

ib_r2c:
        ld      a,b             ; retreive the data char we just got...
	cp	#fesc
        jr      z,ib_r2a        ; if fesc in data stream, switch state

	ld	hl,(in_buffer)
	call	putchar		; shove this character into our buffer
        jr      nc,ib_r2ca

	xor	a
        ld      (in_allo_buf),a
	ld	hl,(in_head)
	call	free_buffer
        jp      ib_rcfend

ib_r2ca:
	ld	(in_buffer),hl	; save in case hl changed
        jp      ib_rc9  ; done so far


; fesc character seen while grabbing data
ib_r2a:
	ld	a,#3
        jp      ib_rcstate

; fend character seen while grabbing data
ib_r2b:
        ld      a,(in_allo_buf)
	or	a
        jr      z,ib_r2z        ; no bytes accumulated, so is null frame

; else we must ship this frame to tx
	ld	hl,(in_buffer)
	call	putchar		; put a garbage character at the end of
				; last buffer because getchar() will strip
				; it. hack needed because of rx use of
				; putchar/getchar.
	ld	hl,(in_head)
        jr      nc,ib_r2za
	call	free_chain
        jr      ib_r2zb
ib_r2za:
        call    tx_queue_ins
ib_r2zb:
	xor	a
        ld      (in_allo_buf),a ; input no longer active
ib_r2z:				; entry point for null frame
	call	sta_off		;done getting this frame, turn sta led off
	ld	a,#1		; keep as was, fends only at end in v.32

        jp      ib_rcstate



; here if we've seen fesc in data stream
ib_r3:
	in	a,(b_dat)
	cp	#tfesc
        jr      z,ib_r3a
	cp	#tfend
        jr      z,ib_r3b

; else we don't know what the hell it is, so ignore & keep collecting bytes
	ld	a,#2
        jp      ib_rcstate

; here if we've seen tfesc after an fesc in data stream; write an fesc
ib_r3a:
	ld	a,#fesc
ib_r3z:
	ld	hl,(in_buffer)
	call	putchar
        jr      nc,ib_r3za

	xor	a
        ld      (in_allo_buf),a
	ld	hl,(in_head)
	call	free_buffer
        jp      ib_rcfend

ib_r3za:
	ld	(in_buffer),hl
	ld	a,#2
        jp      ib_rcstate

; here if we've seen tfend after fesc in data stream; write fend
ib_r3b:
	ld	a,#fend
        jr      ib_r3z          ; rest is same as for tfesc case

; this character is interpreted as txdelay
ib_r10:
	in	a,(b_dat)
	ld	(txdelay),a
        jp      ib_rcfend

; this charcter is p, persistence value
ib_r20:
	in	a,(b_dat)
	ld	(persistence),a
        jp      ib_rcfend

; this character is slottime value
ib_r30:
	in	a,(b_dat)
	ld	(slottime),a
        jp      ib_rcfend


; this character is tailtime value
ib_r40:
	in	a,(b_dat)
	ld	(tailtime),a
        jp      ib_rcfend


; this character is full/half duplex value
; 0 means half duplex, non-zero means full duplex
ib_r50:
	in	a,(b_dat)
	ld	(full_duplex),a
        jp      ib_rcfend


