	.include "tnc.h"
	.area code (rel)

	.globl code_start
code_start:                     ;[dix] CTC init
        ld a,#ctc_vector	;Init interrupt vector
        out (ctc0),a
        ld a,#0b00000101	;no INT, 1/16, TC follows
        out (ctc0),a            ;Channel 0: Modem baud rate * 16
        in a,(switch)
        and #0b00111000		;jumper settings for modem
        rrca
        rrca
        rrca
        ld b,a
        ld a,#1
ctci1:  rlca
        djnz ctci1
        out (ctc0),a

        ld a,#0b00000101        ;no INT, 1/16, TC follows
        out (ctc1),a            ;Channel 1: TTY baud rate * 16
        in a,(switch)
        and #0b00000111         ;jumper settings for TTY
        ld b,a
        ld a,#1
ctci2:  rlca
        djnz ctci2
        out (ctc1),a

        ld a,#0b00100101        ;no INT, 1/256, TC follows
        out (ctc2),a            ;Channel 2: spare channel
        xor a                   ;Slowest count (75 Hz)
        out (ctc2),a

        ld a,#0b10100101        ;INT, 1/256, TC follows
        out (ctc3),a            ;Channel 3: Periodic interrupt every 10 ms
        ld a,#192
        out (ctc3),a            ;100 Hz

;[dix] RAM test (Usefull only with fully decoded memory)

        ld hl,#ram_start
ram1:   ld a,#0x55	;Test pattern 01010101
        ld (hl),a
        cp (hl)
        jr nz,ram2	;if diff there is no more RAM
        ld a,#0xaa	;Test pattern 10101010
        ld (hl),a
        cp (hl)
        jr nz,ram2      ;If diff previous test was lucky passed
        xor a           ;Zero fill
        ld (hl),a
        inc hl          ;Next address
        jr ram1

ram2:   ld sp,hl        ;Stack goes into the end of the RAM
                        ;Now we can use stack related instructions and allow
                        ;interrupts to come.
        push hl         ;Save RAM top for data buffer init

;this sequence loads up our predefined variables into the ram:

	ld	hl,#data_init
	ld	de,#txdelay
	ld	bc,#data_size
	ldir

;[dix] init free buffer chain.

        pop hl		;restore RAM top
        ld de,#bottom   ;and compute the number of avaiable buffers
        or a            ;clear carry
        sbc hl,de       ;Free memory for buffers & stack
        ld b,h          ;b=hl/128
        rlc l
        rl b
        dec b           ;b=buffers (save 128 bytes for stack)
        ld a,b
        ld (nbuffers),a
        dec b           ;b=buffers-1
        ex de,hl        ;beginning of buffer space
                        ;[jdz] now it's also top of free list
ibloop:
	push	hl
	ld	de,#128
	add	hl,de			;hl has "next" pointer
	ex	de,hl			;de has "next" pointer
	pop	hl			;hl now has pointer to current buffer

	ld	(hl),e			;low byte of "next" pointer first
	inc	hl
        ld      (hl),d                  ;now hi byte
	inc	hl
	xor	a
	ld	(hl),a			;zero out count field
	inc	hl
	ld	(hl),a			;zero out # of bytes read field

	ex	de,hl			;hl is now pointer to next buffer
	djnz	ibloop			;and init all the available buffers

	xor	a
	ld	(hl),a			;last "next" address is 0
	inc	hl
	ld	(hl),a			;ditto

	inc	hl
	ld	(hl),a			;zero out count field
	inc	hl
	ld	(hl),a			;zero out # of bytes read field

;------------------------------------------------------------------------------
; Set up the initial message frame [dix]
;------------------------------------------------------------------------------

        call allo_buf                   ;free buffer in hl
        push hl
        ld de,#msg1
        call puts                       ;first text...
        ld ix,#txdelay
        ld b,#6
bsetms: ld a,0(ix)
        inc ix
        call prtn
        call puts
        djnz bsetms
        pop hl
        call out_queue_ins              ;send this chain to TTY


;we re-initialize the sio ports so that we flush garbage chars that may have
;come in while we were diddling the leds.  this is necessary because unless we
;do this, then the a channel (modem) get rx overrun (esp if tnc was listening
;to noise) and rx overrun is very bad - so bad, in fact, that i turn on both
;con and sta and halt, because this situation should never happen in normal
;use.  i flush the b (tty) channel in case anything was sent to it in mid-
;stream.

;re-init sio.

	in	a,(a_ctl)		;assure we are talking to ch 0
	ld	c,#a_ctl
	ld	b,#a_size
	ld	hl,#a_init
	otir				;init sync (modem) port

;re-init async port.

	in	a,(b_ctl)		;assure we are talking to ch 0
	ld	c,#b_ctl
	ld	b,#b_size
	ld	hl,#b_init
	otir				;init async port & interrupt vector

; turn ON interrupts (God save us)

	ei				;let 'em rip!

;-----------------------------------------------------------------------------
; this is the background program.
; note that since everything else is interrupt driven, and saves registers,
; this part of the code can use registers & expect values to stay.

comm_loop:
	call	tx_data
	call	host_tx_data
        jr      comm_loop

;now see if we need to start an output to rs-232 (host) port
host_tx_data:
	ld	a,(out_started)
	or	a			;also clears carry (see below) 
	ret	nz			;if output started, nothing to do

	in	a,(b_ctl)		;look at rr0
	and	#tbe			;isolate the tbe bit
	ret	z

; else we should check to see if we need to start an output
	di
	call	con_off			;
	ld	hl,(out_head_cbuf)	;grab current top of circ buf ptr
	ld	de,(out_tail_cbuf)	;and where the next free buf ptr is
	ei
					;interrupt protect the pickup of the
					;two pointers 3 feb 87
	or	a
	sbc	hl,de
	ret	z			;if the same, nothing to do

;else we need to start an output
	di				;interrupt protect this section,
					;although i'm not sure it needs it...
					;3 feb 87
					;note: it should already be done!
	ld	hl,(out_head_cbuf)	;get pointer to next cbuf to output
	ld	e,(hl)
	inc	hl
	ld	d,(hl)			;de has pointer to buffer chain
        ld      (out_ch_head),de        ;set in interrupt routine's place
	ld	a,#true
	ld	(out_started),a		;yes, output started

	call	con_on	

	ld	a,#fend
	out	(b_dat),a		;send fend character (start txing)
	ei

	ret				;keep looking for new opportunity

tx_data:
	ld	a,(tx_state)
	or	a
        jr      z,txd0
	cp	#1
        jp      z,txd1
	cp	#2
        jp      z,txd2
	cp	#3
        jp      z,txd3
	cp	#4
	ret	z

;when tail timer times out, turn off the tx

	ld	a,(tx_timer)
	or	a
	ret	nz

	ld	a,#5		;ready to write to wr5 of ch a
	di			;must have atomic use of a_wr5 & sio
	out	(a_ctl),a	;next char to a_ctl goes to wr5
	ld	a,(a_wr5)	;grab a_wr5

.if PTT_LOW
        and     #~ rts           ;turn off rts bit there
.else
        or      #rts             ;or turn on (polarity dependent) [dix]
.endif
        ld      (a_wr5),a       ;keep memory copy updated
	out	(a_ctl),a	;and turn off tx now
	xor	a
	ld	(tx_state),a
	ei
	ret

txd0:
        ld      a,(tx_outstng)          ;if there are no outstanding tx...
	or	a			;...frames, then we don't have to...
	ret	z			;...worry about transmitter

; do persistence algorithm
	ld	a,r			;grab the z-80 refresh register
	add	a,a			;double;now 0 <= a reg <= 254
	ld	b,a			;b holds our "random" number
	ld	a,(persistence)
	sub	b			;a reg = persistence - random #
        jr      c,no_ptt                ;if (p-r) < 0 then no ptt now
					; note that p=255 means always key up

; else we've noticed that we've got some frame(s) to send.
; try to keyup tx

	ld	a,(full_duplex)
	or	a
        jr      nz,key_up               ;if full duplex, then there is no
					;need to worry about all this silly
					;slot time and persistence stuff!

;check soft dcd. [dix]
        in      a,(switch)
        bit     7,a
        jr      z,hard_dcd

        ld      a,(rx_state)    ;if rx_state is not 0 we are receiving a frame
        or      a
        jr      nz,no_ptt

        ld      a,(tx_state)    ;if tx_state is not 0 an ia_special irq has
        or      a               ;come and soft_dcd filter is working. [dix]
        jr      z,key_up
        ret

hard_dcd:                               ;[dix]
        ld      a,(a_rr0)               ;a_rr0 is set in interrupt routine
        and     #dcd

.if  DCD_LOW
        jr      nz,no_ptt               ;[jdz] if carrier active, wait it out
.else
        jr      z,no_ptt                ;[dix] polarity dependent
.endif

;ok, so we've won with the random number generator.  keyup tx and start the
;txdelay timer

key_up:
	ld	a,(txdelay)
	ld	(tx_timer),a		;get timer value into timer slot

	ld	a,#5
	di				;we need quite time here.
	out	(a_ctl),a		;ready to write into wr5 of ch a
	ld	a,(a_wr5)

.if PTT_LOW
        or      #rts                     ;turn on the ptt bit...
.else
        and     #~ rts                   ;[dix] polarity dependent
.endif

        ld      (a_wr5),a               ;...in the memory copy of wr5
	out	(a_ctl),a		; keyup transmitter
	ld	a,#2
	ld	(tx_state),a
	ei
	ret				;that's all we do for now, we await
					;txdelay event

no_ptt:	;since we lost on random #, wait slottime before trying again
	ld	a,(slottime)
	ld	(tx_timer),a		;set up the timer value of this event
	ld	a,#1
	ld	(tx_state),a
	ret

txd1:
	ld	a,(tx_timer)
	or	a
	ret	nz
	xor	a
	ld	(tx_state),a
	ret

txd2:
	ld	a,(tx_timer)
	or	a
	ret	nz
	ld	a,#3
	ld	(tx_state),a
	ret

txd3:
	di

	ld	a,#4
	ld	(tx_state),a

	call	txnext_cbuf		;gets hl to point to buffer chain, and
					;sets tx_chain_head for the interrupt
					;routine
	ld	a,#0x80
	out	(a_ctl),a		; reset tx crc
	call	getchar			; getchar needs int. protection
	out	(a_dat),a		; ship this char to tx modem
	ld	a,#true
        ld      (tx_started),a          ; and, yes virgina, we've started tx
	ld	a,#0xc0
	out	(a_ctl),a		; reset tx underrun/eom latch

	ei
	ret

